@@ -20,7 +20,6 @@
 #
 
 from io import BytesIO
-from typing import Union
 from unittest.mock import Mock
 
 from netaddr import IPSet
@@ -58,7 +57,7 @@ class ReadMultipartResponseTests(TestCase):
     redirect_data = b"\r\n\r\n--6067d4698f8d40a0a794ea7d7379d53a\r\nContent-Type: application/json\r\n\r\n{}\r\n--6067d4698f8d40a0a794ea7d7379d53a\r\nLocation: https://cdn.example.org/ab/c1/2345.txt\r\n\r\n--6067d4698f8d40a0a794ea7d7379d53a--\r\n\r\n"
 
     def _build_multipart_response(
-        self, response_length: Union[int, str], max_length: int
+        self, response_length: int | str, max_length: int
     ) -> tuple[
         BytesIO,
         "Deferred[MultipartResponse]",
@@ -208,7 +207,7 @@ class ReadMultipartResponseTests(TestCase):
 
 class ReadBodyWithMaxSizeTests(TestCase):
     def _build_response(
-        self, length: Union[int, str] = UNKNOWN_LENGTH
+        self, length: int | str = UNKNOWN_LENGTH
     ) -> tuple[
         BytesIO,
         "Deferred[int]",
