@@ -19,7 +19,7 @@
 #
 #
 
-from typing import Any, Optional, cast
+from typing import Any, cast
 from unittest.mock import AsyncMock
 
 from parameterized import parameterized
@@ -104,7 +104,7 @@ class ClientIpStoreTestCase(unittest.HomeserverTestCase):
         self.pump(0)
 
         result = cast(
-            list[tuple[str, str, str, Optional[str], int]],
+            list[tuple[str, str, str, str | None, int]],
             self.get_success(
                 self.store.db_pool.simple_select_list(
                     table="user_ips",
@@ -135,7 +135,7 @@ class ClientIpStoreTestCase(unittest.HomeserverTestCase):
         self.pump(0)
 
         result = cast(
-            list[tuple[str, str, str, Optional[str], int]],
+            list[tuple[str, str, str, str | None, int]],
             self.get_success(
                 self.store.db_pool.simple_select_list(
                     table="user_ips",
@@ -184,7 +184,7 @@ class ClientIpStoreTestCase(unittest.HomeserverTestCase):
         else:
             # Check that the new IP and user agent has not been stored yet
             db_result = cast(
-                list[tuple[str, Optional[str], Optional[str], str, Optional[int]]],
+                list[tuple[str, str | None, str | None, str, int | None]],
                 self.get_success(
                     self.store.db_pool.simple_select_list(
                         table="devices",
@@ -266,7 +266,7 @@ class ClientIpStoreTestCase(unittest.HomeserverTestCase):
 
         # Check that the new IP and user agent has not been stored yet
         db_result = cast(
-            list[tuple[str, Optional[str], Optional[str], str, Optional[int]]],
+            list[tuple[str, str | None, str | None, str, int | None]],
             self.get_success(
                 self.store.db_pool.simple_select_list(
                     table="devices",
@@ -589,7 +589,7 @@ class ClientIpStoreTestCase(unittest.HomeserverTestCase):
 
         # We should see that in the DB
         result = cast(
-            list[tuple[str, str, str, Optional[str], int]],
+            list[tuple[str, str, str, str | None, int]],
             self.get_success(
                 self.store.db_pool.simple_select_list(
                     table="user_ips",
@@ -616,7 +616,7 @@ class ClientIpStoreTestCase(unittest.HomeserverTestCase):
 
         # We should get no results.
         result = cast(
-            list[tuple[str, str, str, Optional[str], int]],
+            list[tuple[str, str, str, str | None, int]],
             self.get_success(
                 self.store.db_pool.simple_select_list(
                     table="user_ips",
@@ -695,7 +695,7 @@ class ClientIpStoreTestCase(unittest.HomeserverTestCase):
 
         # We should see that in the DB
         result = cast(
-            list[tuple[str, str, str, Optional[str], int]],
+            list[tuple[str, str, str, str | None, int]],
             self.get_success(
                 self.store.db_pool.simple_select_list(
                     table="user_ips",
