@@ -24,7 +24,6 @@ from typing import (
     Generator,
     Iterable,
     Iterator,
-    Optional,
 )
 from unittest.mock import AsyncMock, Mock
 
@@ -48,12 +47,12 @@ _next_event_id = 1000
 
 
 def create_event(
-    name: Optional[str] = None,
-    type: Optional[str] = None,
-    state_key: Optional[str] = None,
+    name: str | None = None,
+    type: str | None = None,
+    state_key: str | None = None,
     depth: int = 2,
-    event_id: Optional[str] = None,
-    prev_events: Optional[list[tuple[str, dict]]] = None,
+    event_id: str | None = None,
+    prev_events: list[tuple[str, dict]] | None = None,
     **kwargs: Any,
 ) -> EventBase:
     global _next_event_id
@@ -106,7 +105,7 @@ class _DummyStore:
         return groups
 
     async def get_state_ids_for_group(
-        self, state_group: int, state_filter: Optional[StateFilter] = None
+        self, state_group: int, state_filter: StateFilter | None = None
     ) -> MutableStateMap[str]:
         return self._group_to_state[state_group]
 
@@ -114,9 +113,9 @@ class _DummyStore:
         self,
         event_id: str,
         room_id: str,
-        prev_group: Optional[int],
-        delta_ids: Optional[StateMap[str]],
-        current_state_ids: Optional[StateMap[str]],
+        prev_group: int | None,
+        delta_ids: StateMap[str] | None,
+        current_state_ids: StateMap[str] | None,
     ) -> int:
         state_group = self._next_group
         self._next_group += 1
@@ -147,7 +146,7 @@ class _DummyStore:
 
     async def get_state_group_delta(
         self, name: str
-    ) -> tuple[Optional[int], Optional[StateMap[str]]]:
+    ) -> tuple[int | None, StateMap[str] | None]:
         return None, None
 
     def register_events(self, events: Iterable[EventBase]) -> None:
